
From: Colin Leroy <colin@colino.net>

This patch fixes the leak of sb->s_fs_info in both the HFS and HFS+
modules.  In addition to this, it fixes an oops happening when trying to
mount a non-hfsplus filesystem using hfsplus.  This patch is from Roman
Zippel, based off patches sent by myself.

Signed-off-by: Colin Leroy <colin@colino.net>
Signed-off-by: Roman Zippel <zippel@linux-m68k.org>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 fs/hfs/mdb.c       |    5 +++++
 fs/hfs/super.c     |    8 +++-----
 fs/hfsplus/super.c |    6 +++++-
 3 files changed, 13 insertions(+), 6 deletions(-)

diff -puN fs/hfs/mdb.c~hfs-hfsplus-dont-leak-s_fs_info-and-fix-an-oops fs/hfs/mdb.c
--- 25/fs/hfs/mdb.c~hfs-hfsplus-dont-leak-s_fs_info-and-fix-an-oops	Wed Apr 27 15:37:25 2005
+++ 25-akpm/fs/hfs/mdb.c	Wed Apr 27 15:37:25 2005
@@ -333,6 +333,8 @@ void hfs_mdb_close(struct super_block *s
  * Release the resources associated with the in-core MDB.  */
 void hfs_mdb_put(struct super_block *sb)
 {
+	if (!HFS_SB(sb))
+		return;
 	/* free the B-trees */
 	hfs_btree_close(HFS_SB(sb)->ext_tree);
 	hfs_btree_close(HFS_SB(sb)->cat_tree);
@@ -340,4 +342,7 @@ void hfs_mdb_put(struct super_block *sb)
 	/* free the buffers holding the primary and alternate MDBs */
 	brelse(HFS_SB(sb)->mdb_bh);
 	brelse(HFS_SB(sb)->alt_mdb_bh);
+
+	kfree(HFS_SB(sb));
+	sb->s_fs_info = NULL;
 }
diff -puN fs/hfsplus/super.c~hfs-hfsplus-dont-leak-s_fs_info-and-fix-an-oops fs/hfsplus/super.c
--- 25/fs/hfsplus/super.c~hfs-hfsplus-dont-leak-s_fs_info-and-fix-an-oops	Wed Apr 27 15:37:25 2005
+++ 25-akpm/fs/hfsplus/super.c	Wed Apr 27 15:37:25 2005
@@ -208,7 +208,9 @@ static void hfsplus_write_super(struct s
 static void hfsplus_put_super(struct super_block *sb)
 {
 	dprint(DBG_SUPER, "hfsplus_put_super\n");
-	if (!(sb->s_flags & MS_RDONLY)) {
+	if (!sb->s_fs_info)
+		return;
+	if (!(sb->s_flags & MS_RDONLY) && HFSPLUS_SB(sb).s_vhdr) {
 		struct hfsplus_vh *vhdr = HFSPLUS_SB(sb).s_vhdr;
 
 		vhdr->modify_date = hfsp_now2mt();
@@ -226,6 +228,8 @@ static void hfsplus_put_super(struct sup
 	brelse(HFSPLUS_SB(sb).s_vhbh);
 	if (HFSPLUS_SB(sb).nls)
 		unload_nls(HFSPLUS_SB(sb).nls);
+	kfree(sb->s_fs_info);
+	sb->s_fs_info = NULL;
 }
 
 static int hfsplus_statfs(struct super_block *sb, struct kstatfs *buf)
diff -puN fs/hfs/super.c~hfs-hfsplus-dont-leak-s_fs_info-and-fix-an-oops fs/hfs/super.c
--- 25/fs/hfs/super.c~hfs-hfsplus-dont-leak-s_fs_info-and-fix-an-oops	Wed Apr 27 15:37:25 2005
+++ 25-akpm/fs/hfs/super.c	Wed Apr 27 15:37:25 2005
@@ -297,7 +297,7 @@ static int hfs_fill_super(struct super_b
 	res = -EINVAL;
 	if (!parse_options((char *)data, sbi)) {
 		hfs_warn("hfs_fs: unable to parse mount options.\n");
-		goto bail3;
+		goto bail;
 	}
 
 	sb->s_op = &hfs_super_operations;
@@ -310,7 +310,7 @@ static int hfs_fill_super(struct super_b
 			hfs_warn("VFS: Can't find a HFS filesystem on dev %s.\n",
 				hfs_mdb_name(sb));
 		res = -EINVAL;
-		goto bail2;
+		goto bail;
 	}
 
 	/* try to get the root inode */
@@ -340,10 +340,8 @@ bail_iput:
 	iput(root_inode);
 bail_no_root:
 	hfs_warn("hfs_fs: get root inode failed.\n");
+bail:
 	hfs_mdb_put(sb);
-bail2:
-bail3:
-	kfree(sbi);
 	return res;
 }
 
_
